@@ -56,104 +56,106 @@ module test;
   assign _saxi_bready = 1;
   assign _saxi_arcache = 3;
   assign _saxi_arprot = 0;
-  reg [3-1:0] outstanding_wcount_0;
-  wire [32-1:0] _tmp_1;
-  assign _tmp_1 = _saxi_awaddr;
+  reg [3-1:0] __saxi_outstanding_wcount;
+  wire __saxi_has_outstanding_write;
+  assign __saxi_has_outstanding_write = (__saxi_outstanding_wcount > 0) || _saxi_awvalid;
+  wire [32-1:0] _tmp_0;
+  assign _tmp_0 = _saxi_awaddr;
 
   always @(*) begin
-    saxi_awaddr = _tmp_1;
+    saxi_awaddr = _tmp_0;
   end
 
-  wire [4-1:0] _tmp_2;
-  assign _tmp_2 = _saxi_awcache;
+  wire [4-1:0] _tmp_1;
+  assign _tmp_1 = _saxi_awcache;
 
   always @(*) begin
-    saxi_awcache = _tmp_2;
+    saxi_awcache = _tmp_1;
   end
 
-  wire [3-1:0] _tmp_3;
-  assign _tmp_3 = _saxi_awprot;
+  wire [3-1:0] _tmp_2;
+  assign _tmp_2 = _saxi_awprot;
 
   always @(*) begin
-    saxi_awprot = _tmp_3;
+    saxi_awprot = _tmp_2;
   end
 
-  wire _tmp_4;
-  assign _tmp_4 = _saxi_awvalid;
+  wire _tmp_3;
+  assign _tmp_3 = _saxi_awvalid;
 
   always @(*) begin
-    saxi_awvalid = _tmp_4;
+    saxi_awvalid = _tmp_3;
   end
 
   assign _saxi_awready = saxi_awready;
-  wire [32-1:0] _tmp_5;
-  assign _tmp_5 = _saxi_wdata;
+  wire [32-1:0] _tmp_4;
+  assign _tmp_4 = _saxi_wdata;
 
   always @(*) begin
-    saxi_wdata = _tmp_5;
+    saxi_wdata = _tmp_4;
   end
 
-  wire [4-1:0] _tmp_6;
-  assign _tmp_6 = _saxi_wstrb;
+  wire [4-1:0] _tmp_5;
+  assign _tmp_5 = _saxi_wstrb;
 
   always @(*) begin
-    saxi_wstrb = _tmp_6;
+    saxi_wstrb = _tmp_5;
   end
 
-  wire _tmp_7;
-  assign _tmp_7 = _saxi_wvalid;
+  wire _tmp_6;
+  assign _tmp_6 = _saxi_wvalid;
 
   always @(*) begin
-    saxi_wvalid = _tmp_7;
+    saxi_wvalid = _tmp_6;
   end
 
   assign _saxi_wready = saxi_wready;
   assign _saxi_bresp = saxi_bresp;
   assign _saxi_bvalid = saxi_bvalid;
-  wire _tmp_8;
-  assign _tmp_8 = _saxi_bready;
+  wire _tmp_7;
+  assign _tmp_7 = _saxi_bready;
 
   always @(*) begin
-    saxi_bready = _tmp_8;
+    saxi_bready = _tmp_7;
   end
 
-  wire [32-1:0] _tmp_9;
-  assign _tmp_9 = _saxi_araddr;
+  wire [32-1:0] _tmp_8;
+  assign _tmp_8 = _saxi_araddr;
 
   always @(*) begin
-    saxi_araddr = _tmp_9;
+    saxi_araddr = _tmp_8;
   end
 
-  wire [4-1:0] _tmp_10;
-  assign _tmp_10 = _saxi_arcache;
+  wire [4-1:0] _tmp_9;
+  assign _tmp_9 = _saxi_arcache;
 
   always @(*) begin
-    saxi_arcache = _tmp_10;
+    saxi_arcache = _tmp_9;
   end
 
-  wire [3-1:0] _tmp_11;
-  assign _tmp_11 = _saxi_arprot;
+  wire [3-1:0] _tmp_10;
+  assign _tmp_10 = _saxi_arprot;
 
   always @(*) begin
-    saxi_arprot = _tmp_11;
+    saxi_arprot = _tmp_10;
   end
 
-  wire _tmp_12;
-  assign _tmp_12 = _saxi_arvalid;
+  wire _tmp_11;
+  assign _tmp_11 = _saxi_arvalid;
 
   always @(*) begin
-    saxi_arvalid = _tmp_12;
+    saxi_arvalid = _tmp_11;
   end
 
   assign _saxi_arready = saxi_arready;
   assign _saxi_rdata = saxi_rdata;
   assign _saxi_rresp = saxi_rresp;
   assign _saxi_rvalid = saxi_rvalid;
-  wire _tmp_13;
-  assign _tmp_13 = _saxi_rready;
+  wire _tmp_12;
+  assign _tmp_12 = _saxi_rready;
 
   always @(*) begin
-    saxi_rready = _tmp_13;
+    saxi_rready = _tmp_12;
   end
 
   reg [32-1:0] counter;
@@ -172,8 +174,8 @@ module test;
   reg __saxi_cond_5_1;
   reg signed [32-1:0] _th_ctrl_araddr_8;
   reg __saxi_cond_6_1;
-  reg signed [32-1:0] axim_rdata_14;
-  assign _saxi_rready = th_ctrl == 26;
+  reg signed [32-1:0] axim_rdata_13;
+  assign _saxi_rready = th_ctrl == 32;
   reg signed [32-1:0] _th_ctrl_busy_9;
   reg signed [32-1:0] _th_ctrl_end_time_10;
   reg signed [32-1:0] _th_ctrl_time_11;
@@ -209,7 +211,7 @@ module test;
 
 
   initial begin
-    $dumpfile("uut.vcd");
+    $dumpfile("thread_axi_slave_ipxact.vcd");
     $dumpvars(0, uut);
   end
 
@@ -231,7 +233,7 @@ module test;
     _saxi_wvalid = 0;
     _saxi_araddr = 0;
     _saxi_arvalid = 0;
-    outstanding_wcount_0 = 0;
+    __saxi_outstanding_wcount = 0;
     counter = 0;
     th_ctrl = th_ctrl_init;
     _th_ctrl_i_3 = 0;
@@ -247,7 +249,7 @@ module test;
     __saxi_cond_5_1 = 0;
     _th_ctrl_araddr_8 = 0;
     __saxi_cond_6_1 = 0;
-    axim_rdata_14 = 0;
+    axim_rdata_13 = 0;
     _th_ctrl_busy_9 = 0;
     _th_ctrl_end_time_10 = 0;
     _th_ctrl_time_11 = 0;
@@ -262,7 +264,7 @@ module test;
 
   always @(posedge CLK) begin
     if(RST) begin
-      outstanding_wcount_0 <= 0;
+      __saxi_outstanding_wcount <= 0;
       _saxi_awaddr <= 0;
       _saxi_awvalid <= 0;
       __saxi_cond_0_1 <= 0;
@@ -299,13 +301,13 @@ module test;
       if(__saxi_cond_6_1) begin
         _saxi_arvalid <= 0;
       end 
-      if(_saxi_wvalid && _saxi_wready && !(_saxi_bvalid && _saxi_bready) && (outstanding_wcount_0 < 7)) begin
-        outstanding_wcount_0 <= outstanding_wcount_0 + 1;
+      if(_saxi_awvalid && _saxi_awready && !(_saxi_bvalid && _saxi_bready) && (__saxi_outstanding_wcount < 7)) begin
+        __saxi_outstanding_wcount <= __saxi_outstanding_wcount + 1;
       end 
-      if(!(_saxi_wvalid && _saxi_wready) && (_saxi_bvalid && _saxi_bready) && (outstanding_wcount_0 > 0)) begin
-        outstanding_wcount_0 <= outstanding_wcount_0 - 1;
+      if(!(_saxi_awvalid && _saxi_awready) && (_saxi_bvalid && _saxi_bready) && (__saxi_outstanding_wcount > 0)) begin
+        __saxi_outstanding_wcount <= __saxi_outstanding_wcount - 1;
       end 
-      if((th_ctrl == 7) && ((outstanding_wcount_0 < 6) && (_saxi_awready || !_saxi_awvalid))) begin
+      if((th_ctrl == 7) && ((__saxi_outstanding_wcount == 0) && (_saxi_awready || !_saxi_awvalid))) begin
         _saxi_awaddr <= _th_ctrl_awaddr_4;
         _saxi_awvalid <= 1;
       end 
@@ -313,7 +315,7 @@ module test;
       if(_saxi_awvalid && !_saxi_awready) begin
         _saxi_awvalid <= _saxi_awvalid;
       end 
-      if((th_ctrl == 9) && ((outstanding_wcount_0 < 6) && (_saxi_wready || !_saxi_wvalid))) begin
+      if((th_ctrl == 9) && (_saxi_wready || !_saxi_wvalid)) begin
         _saxi_wdata <= _th_ctrl_sleep_5;
         _saxi_wvalid <= 1;
         _saxi_wstrb <= { 4{ 1'd1 } };
@@ -322,7 +324,7 @@ module test;
       if(_saxi_wvalid && !_saxi_wready) begin
         _saxi_wvalid <= _saxi_wvalid;
       end 
-      if((th_ctrl == 13) && ((outstanding_wcount_0 < 6) && (_saxi_awready || !_saxi_awvalid))) begin
+      if((th_ctrl == 15) && ((__saxi_outstanding_wcount == 0) && (_saxi_awready || !_saxi_awvalid))) begin
         _saxi_awaddr <= _th_ctrl_awaddr_4;
         _saxi_awvalid <= 1;
       end 
@@ -330,7 +332,7 @@ module test;
       if(_saxi_awvalid && !_saxi_awready) begin
         _saxi_awvalid <= _saxi_awvalid;
       end 
-      if((th_ctrl == 15) && ((outstanding_wcount_0 < 6) && (_saxi_wready || !_saxi_wvalid))) begin
+      if((th_ctrl == 17) && (_saxi_wready || !_saxi_wvalid)) begin
         _saxi_wdata <= _th_ctrl_size_6;
         _saxi_wvalid <= 1;
         _saxi_wstrb <= { 4{ 1'd1 } };
@@ -339,7 +341,7 @@ module test;
       if(_saxi_wvalid && !_saxi_wready) begin
         _saxi_wvalid <= _saxi_wvalid;
       end 
-      if((th_ctrl == 19) && ((outstanding_wcount_0 < 6) && (_saxi_awready || !_saxi_awvalid))) begin
+      if((th_ctrl == 23) && ((__saxi_outstanding_wcount == 0) && (_saxi_awready || !_saxi_awvalid))) begin
         _saxi_awaddr <= _th_ctrl_awaddr_4;
         _saxi_awvalid <= 1;
       end 
@@ -347,7 +349,7 @@ module test;
       if(_saxi_awvalid && !_saxi_awready) begin
         _saxi_awvalid <= _saxi_awvalid;
       end 
-      if((th_ctrl == 21) && ((outstanding_wcount_0 < 6) && (_saxi_wready || !_saxi_wvalid))) begin
+      if((th_ctrl == 25) && (_saxi_wready || !_saxi_wvalid)) begin
         _saxi_wdata <= 1;
         _saxi_wvalid <= 1;
         _saxi_wstrb <= { 4{ 1'd1 } };
@@ -356,7 +358,7 @@ module test;
       if(_saxi_wvalid && !_saxi_wready) begin
         _saxi_wvalid <= _saxi_wvalid;
       end 
-      if((th_ctrl == 24) && (_saxi_arready || !_saxi_arvalid)) begin
+      if((th_ctrl == 30) && (_saxi_arready || !_saxi_arvalid)) begin
         _saxi_araddr <= _th_ctrl_araddr_8;
         _saxi_arvalid <= 1;
       end 
@@ -411,6 +413,12 @@ module test;
   localparam th_ctrl_33 = 33;
   localparam th_ctrl_34 = 34;
   localparam th_ctrl_35 = 35;
+  localparam th_ctrl_36 = 36;
+  localparam th_ctrl_37 = 37;
+  localparam th_ctrl_38 = 38;
+  localparam th_ctrl_39 = 39;
+  localparam th_ctrl_40 = 40;
+  localparam th_ctrl_41 = 41;
 
   always @(posedge CLK) begin
     if(RST) begin
@@ -421,7 +429,7 @@ module test;
       _th_ctrl_size_6 <= 0;
       _th_ctrl_start_time_7 <= 0;
       _th_ctrl_araddr_8 <= 0;
-      axim_rdata_14 <= 0;
+      axim_rdata_13 <= 0;
       _th_ctrl_busy_9 <= 0;
       _th_ctrl_end_time_10 <= 0;
       _th_ctrl_time_11 <= 0;
@@ -458,12 +466,14 @@ module test;
           th_ctrl <= th_ctrl_7;
         end
         th_ctrl_7: begin
-          if((outstanding_wcount_0 < 6) && (_saxi_awready || !_saxi_awvalid)) begin
+          if((__saxi_outstanding_wcount == 0) && (_saxi_awready || !_saxi_awvalid)) begin
             th_ctrl <= th_ctrl_8;
           end 
         end
         th_ctrl_8: begin
-          th_ctrl <= th_ctrl_9;
+          if(_saxi_awvalid && _saxi_awready) begin
+            th_ctrl <= th_ctrl_9;
+          end 
         end
         th_ctrl_9: begin
           if(_saxi_wready || !_saxi_wvalid) begin
@@ -471,114 +481,150 @@ module test;
           end 
         end
         th_ctrl_10: begin
-          _th_ctrl_awaddr_4 <= 12;
-          th_ctrl <= th_ctrl_11;
+          if(_saxi_wvalid && _saxi_wready) begin
+            th_ctrl <= th_ctrl_11;
+          end 
         end
         th_ctrl_11: begin
-          _th_ctrl_size_6 <= 128;
-          th_ctrl <= th_ctrl_12;
+          if(!__saxi_has_outstanding_write) begin
+            th_ctrl <= th_ctrl_12;
+          end 
         end
         th_ctrl_12: begin
-          $display("# size = %d", _th_ctrl_size_6);
+          _th_ctrl_awaddr_4 <= 12;
           th_ctrl <= th_ctrl_13;
         end
         th_ctrl_13: begin
-          if((outstanding_wcount_0 < 6) && (_saxi_awready || !_saxi_awvalid)) begin
-            th_ctrl <= th_ctrl_14;
-          end 
+          _th_ctrl_size_6 <= 128;
+          th_ctrl <= th_ctrl_14;
         end
         th_ctrl_14: begin
+          $display("# size = %d", _th_ctrl_size_6);
           th_ctrl <= th_ctrl_15;
         end
         th_ctrl_15: begin
-          if(_saxi_wready || !_saxi_wvalid) begin
+          if((__saxi_outstanding_wcount == 0) && (_saxi_awready || !_saxi_awvalid)) begin
             th_ctrl <= th_ctrl_16;
           end 
         end
         th_ctrl_16: begin
-          _th_ctrl_awaddr_4 <= 0;
-          th_ctrl <= th_ctrl_17;
+          if(_saxi_awvalid && _saxi_awready) begin
+            th_ctrl <= th_ctrl_17;
+          end 
         end
         th_ctrl_17: begin
-          _th_ctrl_start_time_7 <= counter;
-          th_ctrl <= th_ctrl_18;
+          if(_saxi_wready || !_saxi_wvalid) begin
+            th_ctrl <= th_ctrl_18;
+          end 
         end
         th_ctrl_18: begin
-          $display("# start time = %d", _th_ctrl_start_time_7);
-          th_ctrl <= th_ctrl_19;
+          if(_saxi_wvalid && _saxi_wready) begin
+            th_ctrl <= th_ctrl_19;
+          end 
         end
         th_ctrl_19: begin
-          if((outstanding_wcount_0 < 6) && (_saxi_awready || !_saxi_awvalid)) begin
+          if(!__saxi_has_outstanding_write) begin
             th_ctrl <= th_ctrl_20;
           end 
         end
         th_ctrl_20: begin
+          _th_ctrl_awaddr_4 <= 0;
           th_ctrl <= th_ctrl_21;
         end
         th_ctrl_21: begin
-          if(_saxi_wready || !_saxi_wvalid) begin
-            th_ctrl <= th_ctrl_22;
-          end 
+          _th_ctrl_start_time_7 <= counter;
+          th_ctrl <= th_ctrl_22;
         end
         th_ctrl_22: begin
-          _th_ctrl_araddr_8 <= 4;
+          $display("# start time = %d", _th_ctrl_start_time_7);
           th_ctrl <= th_ctrl_23;
         end
         th_ctrl_23: begin
-          if(1) begin
+          if((__saxi_outstanding_wcount == 0) && (_saxi_awready || !_saxi_awvalid)) begin
             th_ctrl <= th_ctrl_24;
-          end else begin
-            th_ctrl <= th_ctrl_31;
-          end
+          end 
         end
         th_ctrl_24: begin
-          if(_saxi_arready || !_saxi_arvalid) begin
+          if(_saxi_awvalid && _saxi_awready) begin
             th_ctrl <= th_ctrl_25;
           end 
         end
         th_ctrl_25: begin
-          th_ctrl <= th_ctrl_26;
+          if(_saxi_wready || !_saxi_wvalid) begin
+            th_ctrl <= th_ctrl_26;
+          end 
         end
         th_ctrl_26: begin
-          if(_saxi_rvalid) begin
-            axim_rdata_14 <= _saxi_rdata;
-          end 
-          if(_saxi_rvalid) begin
+          if(_saxi_wvalid && _saxi_wready) begin
             th_ctrl <= th_ctrl_27;
           end 
         end
         th_ctrl_27: begin
-          _th_ctrl_busy_9 <= axim_rdata_14;
-          th_ctrl <= th_ctrl_28;
+          if(!__saxi_has_outstanding_write) begin
+            th_ctrl <= th_ctrl_28;
+          end 
         end
         th_ctrl_28: begin
-          if(!_th_ctrl_busy_9) begin
-            th_ctrl <= th_ctrl_29;
-          end else begin
-            th_ctrl <= th_ctrl_30;
-          end
+          _th_ctrl_araddr_8 <= 4;
+          th_ctrl <= th_ctrl_29;
         end
         th_ctrl_29: begin
-          th_ctrl <= th_ctrl_31;
+          if(1) begin
+            th_ctrl <= th_ctrl_30;
+          end else begin
+            th_ctrl <= th_ctrl_37;
+          end
         end
         th_ctrl_30: begin
-          th_ctrl <= th_ctrl_23;
+          if(_saxi_arready || !_saxi_arvalid) begin
+            th_ctrl <= th_ctrl_31;
+          end 
         end
         th_ctrl_31: begin
-          _th_ctrl_end_time_10 <= counter;
-          th_ctrl <= th_ctrl_32;
+          if(_saxi_arvalid && _saxi_arready) begin
+            th_ctrl <= th_ctrl_32;
+          end 
         end
         th_ctrl_32: begin
-          $display("# end time = %d", _th_ctrl_end_time_10);
-          th_ctrl <= th_ctrl_33;
+          if(_saxi_rvalid) begin
+            axim_rdata_13 <= _saxi_rdata;
+          end 
+          if(_saxi_rvalid) begin
+            th_ctrl <= th_ctrl_33;
+          end 
         end
         th_ctrl_33: begin
-          _th_ctrl_time_11 <= _th_ctrl_end_time_10 - _th_ctrl_start_time_7;
+          _th_ctrl_busy_9 <= axim_rdata_13;
           th_ctrl <= th_ctrl_34;
         end
         th_ctrl_34: begin
+          if(!_th_ctrl_busy_9) begin
+            th_ctrl <= th_ctrl_35;
+          end else begin
+            th_ctrl <= th_ctrl_36;
+          end
+        end
+        th_ctrl_35: begin
+          th_ctrl <= th_ctrl_37;
+        end
+        th_ctrl_36: begin
+          th_ctrl <= th_ctrl_29;
+        end
+        th_ctrl_37: begin
+          _th_ctrl_end_time_10 <= counter;
+          th_ctrl <= th_ctrl_38;
+        end
+        th_ctrl_38: begin
+          $display("# end time = %d", _th_ctrl_end_time_10);
+          th_ctrl <= th_ctrl_39;
+        end
+        th_ctrl_39: begin
+          _th_ctrl_time_11 <= _th_ctrl_end_time_10 - _th_ctrl_start_time_7;
+          th_ctrl <= th_ctrl_40;
+        end
+        th_ctrl_40: begin
           $display("# exec time = %d", _th_ctrl_time_11);
-          th_ctrl <= th_ctrl_35;
+          th_ctrl <= th_ctrl_41;
         end
       endcase
     end
@@ -660,7 +706,7 @@ module blinkled
                                (axis_maskaddr_5 == 2)? _saxi_resetval_2 : 
                                (axis_maskaddr_5 == 3)? _saxi_resetval_3 : 'hx;
   reg _saxi_cond_0_1;
-  assign saxi_wready = _saxi_register_fsm == 2;
+  assign saxi_wready = _saxi_register_fsm == 3;
   reg [32-1:0] th_blink;
   localparam th_blink_init = 0;
   reg signed [32-1:0] _th_blink_size_0;
@@ -732,16 +778,16 @@ module blinkled
         _saxi_register_3 <= axislite_resetval_8;
         _saxi_flag_3 <= 0;
       end 
-      if((_saxi_register_fsm == 2) && saxi_wvalid && (axis_maskaddr_5 == 0)) begin
+      if((_saxi_register_fsm == 3) && saxi_wvalid && (axis_maskaddr_5 == 0)) begin
         _saxi_register_0 <= saxi_wdata;
       end 
-      if((_saxi_register_fsm == 2) && saxi_wvalid && (axis_maskaddr_5 == 1)) begin
+      if((_saxi_register_fsm == 3) && saxi_wvalid && (axis_maskaddr_5 == 1)) begin
         _saxi_register_1 <= saxi_wdata;
       end 
-      if((_saxi_register_fsm == 2) && saxi_wvalid && (axis_maskaddr_5 == 2)) begin
+      if((_saxi_register_fsm == 3) && saxi_wvalid && (axis_maskaddr_5 == 2)) begin
         _saxi_register_2 <= saxi_wdata;
       end 
-      if((_saxi_register_fsm == 2) && saxi_wvalid && (axis_maskaddr_5 == 3)) begin
+      if((_saxi_register_fsm == 3) && saxi_wvalid && (axis_maskaddr_5 == 3)) begin
         _saxi_register_3 <= saxi_wdata;
       end 
       if((_saxi_register_0 == 1) && (th_blink == 2) && 1) begin
@@ -793,6 +839,8 @@ module blinkled
 
   localparam _saxi_register_fsm_1 = 1;
   localparam _saxi_register_fsm_2 = 2;
+  localparam _saxi_register_fsm_3 = 3;
+  localparam _saxi_register_fsm_4 = 4;
 
   always @(posedge CLK) begin
     if(RST) begin
@@ -808,16 +856,26 @@ module blinkled
             _saxi_register_fsm <= _saxi_register_fsm_1;
           end 
           if(writevalid_1) begin
-            _saxi_register_fsm <= _saxi_register_fsm_2;
+            _saxi_register_fsm <= _saxi_register_fsm_3;
           end 
         end
         _saxi_register_fsm_1: begin
           if(saxi_rready || !saxi_rvalid) begin
-            _saxi_register_fsm <= _saxi_register_fsm_init;
+            _saxi_register_fsm <= _saxi_register_fsm_2;
           end 
         end
         _saxi_register_fsm_2: begin
+          if(saxi_rready && saxi_rvalid) begin
+            _saxi_register_fsm <= _saxi_register_fsm_init;
+          end 
+        end
+        _saxi_register_fsm_3: begin
           if(saxi_wvalid) begin
+            _saxi_register_fsm <= _saxi_register_fsm_4;
+          end 
+        end
+        _saxi_register_fsm_4: begin
+          if(saxi_bready && saxi_bvalid) begin
             _saxi_register_fsm <= _saxi_register_fsm_init;
           end 
         end
