@@ -42,7 +42,8 @@ def mkTest():
     clk = uut['CLK']
     rst = uut['RST']
 
-    simulation.setup_waveform(m, uut, m.get_vars())
+    vcd_name = os.path.splitext(os.path.basename(__file__))[0] + '.vcd'
+    simulation.setup_waveform(m, uut, m.get_vars(), dumpfile=vcd_name)
     simulation.setup_clock(m, clk, hperiod=5)
     init = simulation.setup_reset(m, rst, m.make_reset(), period=100)
 
