@@ -4,10 +4,12 @@ import sys
 import os
 
 # the next line can be removed after installation
-sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))))
+sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(
+    os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))))
 
 from veriloggen import *
 
+
 def mkLed():
     m = Module('blinkled')
     width = m.Parameter('WIDTH', 8)
@@ -20,20 +22,21 @@ def mkLed():
     # get the initial index (= 0)
     init = fsm.current
 
-    fsm( count(count + 1) )
+    fsm(count(count + 1))
     fsm.goto_next()
-    fsm( count(count + 2) )
+    fsm(count(count + 2))
     fsm.goto_next()
-    
+
     # get the current index (= 2)
     here = fsm.current
-    fsm( count(count + 3) )
+    fsm(count(count + 3))
     fsm.goto_next()
 
     # jump by using label "init"
-    fsm.goto( dst=init, cond=(count < 1024), else_dst=fsm.next ).inc()
-    
-    fsm( led(led + 1) )
+    fsm.goto(dst=init, cond=(count < 1024), else_dst=fsm.next)
+    fsm.inc()
+
+    fsm(led(led + 1))
     # jump by using label "here"
     fsm.goto(here)
 
@@ -41,6 +44,7 @@ def mkLed():
 
     return m
 
+
 if __name__ == '__main__':
     led = mkLed()
     verilog = led.to_verilog()
