@@ -122,8 +122,8 @@ def mkLed():
     return m
 
 
-def mkTest(memimg_name=None):
-    m = Module('test')
+def mkTestTop(memimg_name=None):
+    m = Module('test_top')
 
     # target instance
     led = mkLed()
@@ -182,7 +182,8 @@ def mkTest(memimg_name=None):
                      params=m.connect_params(led),
                      ports=m.connect_ports(led))
 
-    # simulation.setup_waveform(m, uut)
+    # vcd_name = os.path.splitext(os.path.basename(__file__))[0] + '.vcd'
+    # simulation.setup_waveform(m, uut, dumpfile=vcd_name)
     simulation.setup_clock(m, clk, hperiod=5)
     init = simulation.setup_reset(m, rst, m.make_reset(), period=100)
 
@@ -194,6 +195,23 @@ def mkTest(memimg_name=None):
     return m
 
 
+def mkTest(memimg_name=None):
+
+    m = Module('test')
+
+    # target instance
+    test = mkTestTop(memimg_name)
+    params = m.copy_params(test)
+    ports = m.copy_sim_ports(test)
+    uut = m.Instance(test, 'uut',
+                     params=m.connect_params(test),
+                     ports=m.connect_ports(test))
+    # vcd_name = os.path.splitext(os.path.basename(__file__))[0] + '.vcd'
+    # simulation.setup_waveform(m, uut, dumpfile=vcd_name)
+
+    return m
+
+
 def run(filename='tmp.v', simtype='iverilog', outputfile=None):
 
     if outputfile is None:
