@@ -91,9 +91,11 @@ module blinkled
   reg [32-1:0] _axi_a_read_local_addr_buf;
   reg [32-1:0] _axi_a_read_local_stride_buf;
   reg [33-1:0] _axi_a_read_local_size_buf;
-  reg _axi_a_read_data_idle;
+  reg _axi_a_read_data_busy;
+  wire _axi_a_read_data_idle;
   wire _axi_a_read_idle;
-  assign _axi_a_read_idle = _axi_a_read_req_fifo_empty && _axi_a_read_data_idle;
+  assign _axi_a_read_data_idle = _axi_a_read_req_fifo_empty && !_axi_a_read_data_busy;
+  assign _axi_a_read_idle = _axi_a_read_data_idle;
   wire _axi_b_read_req_fifo_enq;
   wire [105-1:0] _axi_b_read_req_fifo_wdata;
   wire _axi_b_read_req_fifo_full;
@@ -139,9 +141,11 @@ module blinkled
   reg [32-1:0] _axi_b_read_local_addr_buf;
   reg [32-1:0] _axi_b_read_local_stride_buf;
   reg [33-1:0] _axi_b_read_local_size_buf;
-  reg _axi_b_read_data_idle;
+  reg _axi_b_read_data_busy;
+  wire _axi_b_read_data_idle;
   wire _axi_b_read_idle;
-  assign _axi_b_read_idle = _axi_b_read_req_fifo_empty && _axi_b_read_data_idle;
+  assign _axi_b_read_data_idle = _axi_b_read_req_fifo_empty && !_axi_b_read_data_busy;
+  assign _axi_b_read_idle = _axi_b_read_data_idle;
   wire _axi_c_write_req_fifo_enq;
   wire [105-1:0] _axi_c_write_req_fifo_wdata;
   wire _axi_c_write_req_fifo_full;
@@ -187,9 +191,11 @@ module blinkled
   reg [32-1:0] _axi_c_write_local_addr_buf;
   reg [32-1:0] _axi_c_write_local_stride_buf;
   reg [33-1:0] _axi_c_write_size_buf;
-  reg _axi_c_write_data_idle;
+  reg _axi_c_write_data_busy;
+  wire _axi_c_write_data_idle;
   wire _axi_c_write_idle;
-  assign _axi_c_write_idle = _axi_c_write_req_fifo_empty && _axi_c_write_data_idle;
+  assign _axi_c_write_data_idle = _axi_c_write_req_fifo_empty && !_axi_c_write_data_busy;
+  assign _axi_c_write_idle = _axi_c_write_data_idle;
   assign saxi_bresp = 0;
   assign saxi_rresp = 0;
   reg signed [32-1:0] _saxi_register_0;
@@ -233,7 +239,7 @@ module blinkled
                                 (axis_maskaddr_17 == 2)? _saxi_resetval_2 : 
                                 (axis_maskaddr_17 == 3)? _saxi_resetval_3 : 'hx;
   reg _saxi_cond_0_1;
-  assign saxi_wready = _saxi_register_fsm == 2;
+  assign saxi_wready = _saxi_register_fsm == 3;
   wire [10-1:0] ram_a_0_addr;
   wire [32-1:0] ram_a_0_rdata;
   wire [32-1:0] ram_a_0_wdata;
@@ -411,7 +417,7 @@ module blinkled
   reg [_tmp_26-1:0] __tmp_27_1;
   reg [32-1:0] _axi_a_read_data_fsm;
   localparam _axi_a_read_data_fsm_init = 0;
-  assign _axi_a_read_req_fifo_deq = ((_axi_a_read_data_fsm == 0) && (_axi_a_read_data_idle && !_axi_a_read_req_fifo_empty && (_axi_a_read_op_sel_fifo == 1)) && !_axi_a_read_req_fifo_empty)? 1 : 0;
+  assign _axi_a_read_req_fifo_deq = ((_axi_a_read_data_fsm == 0) && (!_axi_a_read_data_busy && !_axi_a_read_req_fifo_empty && (_axi_a_read_op_sel_fifo == 1)) && !_axi_a_read_req_fifo_empty)? 1 : 0;
   reg [32-1:0] write_burst_fsm_0;
   localparam write_burst_fsm_0_init = 0;
   reg [10-1:0] write_burst_addr_28;
@@ -441,7 +447,7 @@ module blinkled
   reg [_tmp_37-1:0] __tmp_38_1;
   reg [32-1:0] _axi_b_read_data_fsm;
   localparam _axi_b_read_data_fsm_init = 0;
-  assign _axi_b_read_req_fifo_deq = ((_axi_b_read_data_fsm == 0) && (_axi_b_read_data_idle && !_axi_b_read_req_fifo_empty && (_axi_b_read_op_sel_fifo == 1)) && !_axi_b_read_req_fifo_empty)? 1 : 0;
+  assign _axi_b_read_req_fifo_deq = ((_axi_b_read_data_fsm == 0) && (!_axi_b_read_data_busy && !_axi_b_read_req_fifo_empty && (_axi_b_read_op_sel_fifo == 1)) && !_axi_b_read_req_fifo_empty)? 1 : 0;
   reg [32-1:0] write_burst_fsm_1;
   localparam write_burst_fsm_1_init = 0;
   reg [10-1:0] write_burst_addr_39;
@@ -547,7 +553,7 @@ module blinkled
   reg [_tmp_81-1:0] __tmp_82_1;
   reg [32-1:0] _axi_c_write_data_fsm;
   localparam _axi_c_write_data_fsm_init = 0;
-  assign _axi_c_write_req_fifo_deq = ((_axi_c_write_data_fsm == 0) && (_axi_c_write_data_idle && !_axi_c_write_req_fifo_empty && (_axi_c_write_op_sel_fifo == 1)) && !_axi_c_write_req_fifo_empty)? 1 : 0;
+  assign _axi_c_write_req_fifo_deq = ((_axi_c_write_data_fsm == 0) && (!_axi_c_write_data_busy && !_axi_c_write_req_fifo_empty && (_axi_c_write_op_sel_fifo == 1)) && !_axi_c_write_req_fifo_empty)? 1 : 0;
   reg [32-1:0] read_burst_fsm_2;
   localparam read_burst_fsm_2_init = 0;
   reg [10-1:0] read_burst_addr_83;
@@ -567,14 +573,14 @@ module blinkled
 
   always @(posedge CLK) begin
     if(RST) begin
-      _axi_a_read_data_idle <= 1;
+      _axi_a_read_data_busy <= 0;
       _axi_a_read_op_sel_buf <= 0;
       _axi_a_read_local_addr_buf <= 0;
       _axi_a_read_local_stride_buf <= 0;
       _axi_a_read_local_size_buf <= 0;
     end else begin
-      if((_axi_a_read_data_fsm == 0) && (_axi_a_read_data_idle && !_axi_a_read_req_fifo_empty && (_axi_a_read_op_sel_fifo == 1))) begin
-        _axi_a_read_data_idle <= 0;
+      if((_axi_a_read_data_fsm == 0) && (!_axi_a_read_data_busy && !_axi_a_read_req_fifo_empty && (_axi_a_read_op_sel_fifo == 1))) begin
+        _axi_a_read_data_busy <= 1;
         _axi_a_read_op_sel_buf <= _axi_a_read_op_sel_fifo;
         _axi_a_read_local_addr_buf <= _axi_a_read_local_addr_fifo;
         _axi_a_read_local_stride_buf <= _axi_a_read_local_stride_fifo;
@@ -584,7 +590,7 @@ module blinkled
         _axi_a_read_local_size_buf <= _axi_a_read_local_size_buf - 1;
       end 
       if((_axi_a_read_data_fsm == 2) && axi_a_tvalid && (_axi_a_read_local_size_buf <= 1)) begin
-        _axi_a_read_data_idle <= 1;
+        _axi_a_read_data_busy <= 0;
       end 
     end
   end
@@ -609,14 +615,14 @@ module blinkled
 
   always @(posedge CLK) begin
     if(RST) begin
-      _axi_b_read_data_idle <= 1;
+      _axi_b_read_data_busy <= 0;
       _axi_b_read_op_sel_buf <= 0;
       _axi_b_read_local_addr_buf <= 0;
       _axi_b_read_local_stride_buf <= 0;
       _axi_b_read_local_size_buf <= 0;
     end else begin
-      if((_axi_b_read_data_fsm == 0) && (_axi_b_read_data_idle && !_axi_b_read_req_fifo_empty && (_axi_b_read_op_sel_fifo == 1))) begin
-        _axi_b_read_data_idle <= 0;
+      if((_axi_b_read_data_fsm == 0) && (!_axi_b_read_data_busy && !_axi_b_read_req_fifo_empty && (_axi_b_read_op_sel_fifo == 1))) begin
+        _axi_b_read_data_busy <= 1;
         _axi_b_read_op_sel_buf <= _axi_b_read_op_sel_fifo;
         _axi_b_read_local_addr_buf <= _axi_b_read_local_addr_fifo;
         _axi_b_read_local_stride_buf <= _axi_b_read_local_stride_fifo;
@@ -626,7 +632,7 @@ module blinkled
         _axi_b_read_local_size_buf <= _axi_b_read_local_size_buf - 1;
       end 
       if((_axi_b_read_data_fsm == 2) && axi_b_tvalid && (_axi_b_read_local_size_buf <= 1)) begin
-        _axi_b_read_data_idle <= 1;
+        _axi_b_read_data_busy <= 0;
       end 
     end
   end
@@ -651,7 +657,7 @@ module blinkled
 
   always @(posedge CLK) begin
     if(RST) begin
-      _axi_c_write_data_idle <= 1;
+      _axi_c_write_data_busy <= 0;
       _axi_c_write_op_sel_buf <= 0;
       _axi_c_write_local_addr_buf <= 0;
       _axi_c_write_local_stride_buf <= 0;
@@ -665,8 +671,8 @@ module blinkled
         axi_c_tvalid <= 0;
         axi_c_tlast <= 0;
       end 
-      if((_axi_c_write_data_fsm == 0) && (_axi_c_write_data_idle && !_axi_c_write_req_fifo_empty && (_axi_c_write_op_sel_fifo == 1))) begin
-        _axi_c_write_data_idle <= 0;
+      if((_axi_c_write_data_fsm == 0) && (!_axi_c_write_data_busy && !_axi_c_write_req_fifo_empty && (_axi_c_write_op_sel_fifo == 1))) begin
+        _axi_c_write_data_busy <= 1;
         _axi_c_write_op_sel_buf <= _axi_c_write_op_sel_fifo;
         _axi_c_write_local_addr_buf <= _axi_c_write_local_addr_fifo;
         _axi_c_write_local_stride_buf <= _axi_c_write_local_stride_fifo;
@@ -686,7 +692,7 @@ module blinkled
         _axi_c_write_size_buf <= _axi_c_write_size_buf - 1;
       end 
       if((_axi_c_write_data_fsm == 2) && ((_axi_c_write_op_sel_buf == 1) && read_burst_rvalid_86 && (axi_c_tready || !axi_c_tvalid)) && read_burst_rlast_87) begin
-        _axi_c_write_data_idle <= 1;
+        _axi_c_write_data_busy <= 0;
       end 
     end
   end
@@ -773,16 +779,16 @@ module blinkled
         _saxi_register_3 <= axislite_resetval_20;
         _saxi_flag_3 <= 0;
       end 
-      if((_saxi_register_fsm == 2) && saxi_wvalid && (axis_maskaddr_17 == 0)) begin
+      if((_saxi_register_fsm == 3) && saxi_wvalid && (axis_maskaddr_17 == 0)) begin
         _saxi_register_0 <= saxi_wdata;
       end 
-      if((_saxi_register_fsm == 2) && saxi_wvalid && (axis_maskaddr_17 == 1)) begin
+      if((_saxi_register_fsm == 3) && saxi_wvalid && (axis_maskaddr_17 == 1)) begin
         _saxi_register_1 <= saxi_wdata;
       end 
-      if((_saxi_register_fsm == 2) && saxi_wvalid && (axis_maskaddr_17 == 2)) begin
+      if((_saxi_register_fsm == 3) && saxi_wvalid && (axis_maskaddr_17 == 2)) begin
         _saxi_register_2 <= saxi_wdata;
       end 
-      if((_saxi_register_fsm == 2) && saxi_wvalid && (axis_maskaddr_17 == 3)) begin
+      if((_saxi_register_fsm == 3) && saxi_wvalid && (axis_maskaddr_17 == 3)) begin
         _saxi_register_3 <= saxi_wdata;
       end 
       if((_saxi_register_0 == 1) && (th_comp == 2) && 1) begin
@@ -834,6 +840,8 @@ module blinkled
 
   localparam _saxi_register_fsm_1 = 1;
   localparam _saxi_register_fsm_2 = 2;
+  localparam _saxi_register_fsm_3 = 3;
+  localparam _saxi_register_fsm_4 = 4;
 
   always @(posedge CLK) begin
     if(RST) begin
@@ -849,16 +857,26 @@ module blinkled
             _saxi_register_fsm <= _saxi_register_fsm_1;
           end 
           if(writevalid_13) begin
-            _saxi_register_fsm <= _saxi_register_fsm_2;
+            _saxi_register_fsm <= _saxi_register_fsm_3;
           end 
         end
         _saxi_register_fsm_1: begin
           if(saxi_rready || !saxi_rvalid) begin
-            _saxi_register_fsm <= _saxi_register_fsm_init;
+            _saxi_register_fsm <= _saxi_register_fsm_2;
           end 
         end
         _saxi_register_fsm_2: begin
+          if(saxi_rready && saxi_rvalid) begin
+            _saxi_register_fsm <= _saxi_register_fsm_init;
+          end 
+        end
+        _saxi_register_fsm_3: begin
           if(saxi_wvalid) begin
+            _saxi_register_fsm <= _saxi_register_fsm_4;
+          end 
+        end
+        _saxi_register_fsm_4: begin
+          if(saxi_bready && saxi_bvalid) begin
             _saxi_register_fsm <= _saxi_register_fsm_init;
           end 
         end
@@ -1356,7 +1374,7 @@ module blinkled
     end else begin
       case(_axi_a_read_data_fsm)
         _axi_a_read_data_fsm_init: begin
-          if(_axi_a_read_data_idle && !_axi_a_read_req_fifo_empty && (_axi_a_read_op_sel_fifo == 1)) begin
+          if(!_axi_a_read_data_busy && !_axi_a_read_req_fifo_empty && (_axi_a_read_op_sel_fifo == 1)) begin
             _axi_a_read_data_fsm <= _axi_a_read_data_fsm_1;
           end 
         end
@@ -1427,7 +1445,7 @@ module blinkled
     end else begin
       case(_axi_b_read_data_fsm)
         _axi_b_read_data_fsm_init: begin
-          if(_axi_b_read_data_idle && !_axi_b_read_req_fifo_empty && (_axi_b_read_op_sel_fifo == 1)) begin
+          if(!_axi_b_read_data_busy && !_axi_b_read_req_fifo_empty && (_axi_b_read_op_sel_fifo == 1)) begin
             _axi_b_read_data_fsm <= _axi_b_read_data_fsm_1;
           end 
         end
@@ -1588,7 +1606,7 @@ module blinkled
     end else begin
       case(_axi_c_write_data_fsm)
         _axi_c_write_data_fsm_init: begin
-          if(_axi_c_write_data_idle && !_axi_c_write_req_fifo_empty && (_axi_c_write_op_sel_fifo == 1)) begin
+          if(!_axi_c_write_data_busy && !_axi_c_write_req_fifo_empty && (_axi_c_write_op_sel_fifo == 1)) begin
             _axi_c_write_data_fsm <= _axi_c_write_data_fsm_1;
           end 
         end
