@@ -17,51 +17,83 @@ def mkMain():
     rst = m.Input('RST')
 
     myaxi = axi.AxiMaster(m, 'myaxi', clk, rst)
-    myaxi.disable_read()
 
     fsm = FSM(m, 'fsm', clk, rst)
 
-    # write address (1)
-    awaddr = 1024
-    awlen = 64
-
-    ack, counter = myaxi.write_request_counter(awaddr, awlen, cond=fsm)
+    # write request (1)
+    awaddr1 = 1024
+    awlen1 = 64
+    ack = myaxi.write_request(awaddr1, awlen1, cond=fsm)
     fsm.If(ack).goto_next()
 
     # write data (1)
-    wdata = m.Reg('wdata', 32, initval=0)
-
-    ack, last = myaxi.write_data(wdata, counter, cond=fsm)
+    wdata1 = m.Reg('wdata1', 32, initval=0)
+    wlast1 = wdata1 == awlen1 - 1
+    ack = myaxi.write_data(wdata1, wlast1, cond=fsm)
 
     fsm.If(ack)(
-        wdata.inc()
+        wdata1.inc()
     )
-    fsm.If(last).goto_next()
+    fsm.If(ack, wlast1).goto_next()
 
-    # write address (2)
-    ack, counter = myaxi.write_request_counter(awaddr, awlen, cond=fsm)
+    # write request (2)
+    awaddr2 = 1024 + 1024
+    awlen2 = 64 + 64
+    ack = myaxi.write_request(awaddr2, awlen2, cond=fsm)
     fsm.If(ack).goto_next()
 
     # write data (2)
-    ack, last = myaxi.write_data(wdata, counter, cond=fsm)
+    wdata2 = m.Reg('wdata2', 32, initval=0)
+    wlast2 = wdata2 == awlen2 - 1
+    ack = myaxi.write_data(wdata2, wlast2, cond=fsm)
 
     fsm.If(ack)(
-        wdata.inc()
+        wdata2.inc()
     )
-    fsm.If(last).goto_next()
+    fsm.If(ack, wlast2).goto_next()
+    fsm.If(Not(myaxi.wdata.wvalid)).goto_next()
 
-    # check
-    sum = m.Reg('sum', 32, initval=0)
-    expected_sum = (awlen * 2 - 1) * (awlen * 2) // 2
+    # read request (1)
+    araddr1 = awaddr1
+    arlen1 = awlen1
+    ack = myaxi.read_request(araddr1, arlen1, cond=fsm)
+    fsm.If(ack).goto_next()
 
-    seq = Seq(m, 'seq', clk, rst)
-    seq.If(Ands(myaxi.wdata.wvalid, myaxi.wdata.wready))(
-        sum.add(myaxi.wdata.wdata)
+    # read data (1)
+    data, valid, last = myaxi.read_data(cond=fsm)
+    sum = m.Reg('sum', width=32, initval=0)
+
+    fsm.If(valid)(
+        sum.add(data)
     )
-    seq.Then().If(myaxi.wdata.wlast).Delay(1)(
-        Systask('display', "current_sum=%d expected_sum=%d", sum, expected_sum),
-        If(fsm.here)(If(NotEql(sum, expected_sum))(Display('# verify: FAILED')).Else(Display('# verify: PASSED')))
+    fsm.If(valid, last).goto_next()
+
+    # read request (2)
+    araddr2 = awaddr2
+    arlen2 = awlen2
+    ack = myaxi.read_request(araddr2, arlen2, cond=fsm)
+    fsm.If(ack).goto_next()
+
+    # read data (2)
+    data, valid, last = myaxi.read_data(cond=fsm)
+
+    fsm.If(valid)(
+        sum.add(data)
     )
+    fsm.If(valid, last).goto_next()
+
+    # verify
+    expected_sum = (((0 + awlen1 - 1) * awlen1) // 2 +
+                    ((0 + awlen2 - 1) * awlen2) // 2)
+    fsm(
+        Systask('display', 'sum=%d expected_sum=%d', sum, expected_sum)
+    )
+    fsm.If(sum == expected_sum)(
+        Systask('display', '# verify: PASSED')
+    ).Else(
+        Systask('display', '# verify: FAILED')
+    )
+    fsm.goto_next()
 
     return m
 
@@ -79,7 +111,7 @@ def mkTest(memimg_name=None):
     clk = ports['CLK']
     rst = ports['RST']
 
-    memory = axi.AxiMemoryModel(m, 'memory', clk, rst)
+    memory = axi.AxiMemoryModel(m, 'memory', clk, rst, memimg_name=memimg_name)
     memory.connect(ports, 'myaxi')
 
     uut = m.Instance(main, 'uut',
