@@ -37,7 +37,14 @@ def setup_waveform(m, *uuts, **kwargs):
                 continue
             new_uuts.append(uut)
 
-    dumpfile = kwargs['dumpfile'] if 'dumpfile' in kwargs else 'uut.vcd'
+    if 'dumpfile' in kwargs:
+        dumpfile = kwargs['dumpfile']
+    else:
+        dumpfile_fd = tempfile.NamedTemporaryFile(prefix="waveform_",
+                                                  suffix=".vcd",
+                                                  dir=os.getcwd(),
+                                                  delete=False)
+        dumpfile = dumpfile_fd.name
 
     uuts = new_uuts
     ret = m.Initial(
@@ -128,8 +135,14 @@ def run_iverilog(objs, display=False, top=None, outputfile=None,
 
     top_name = top.name if isinstance(top, module.Module) else top
 
+    tmp_outputfile_fd = None
     if outputfile is None:
-        outputfile = 'a.out'
+        tmp_outputfile_fd = tempfile.NamedTemporaryFile(prefix="iverilog_",
+                                                        suffix=".out",
+                                                        dir=os.getcwd())
+        outputfile = tmp_outputfile_fd.name
+
+    outputfile = os.path.abspath(outputfile)
 
     cmd = []
     cmd.append('iverilog')
@@ -193,6 +206,9 @@ def run_iverilog(objs, display=False, top=None, outputfile=None,
     if os.name == 'nt':
         os.remove(path)
 
+    if tmp_outputfile_fd is not None:
+        tmp_outputfile_fd.close()
+
     return ''.join([syn_rslt, sim_rslt])
 
 
@@ -208,8 +224,14 @@ def run_vcs(objs, display=False, top=None, outputfile=None,
 
     top_name = top.name if isinstance(top, module.Module) else top
 
+    tmp_outputfile_fd = None
     if outputfile is None:
-        outputfile = 'simv'
+        tmp_outputfile_fd = tempfile.NamedTemporaryFile(prefix="vcs_",
+                                                        suffix=".out",
+                                                        dir=os.getcwd())
+        outputfile = tmp_outputfile_fd.name
+
+    outputfile = os.path.abspath(outputfile)
 
     cmd = []
     cmd.append('vcs')
@@ -272,7 +294,7 @@ def run_vcs(objs, display=False, top=None, outputfile=None,
     if os.name == 'nt':
         cmd.append(outputfile)
     else:
-        cmd.append('./' + outputfile)
+        cmd.append(outputfile)
 
     sim_rslt = _exec(' '.join(cmd), encoding, display)
 
@@ -282,6 +304,9 @@ def run_vcs(objs, display=False, top=None, outputfile=None,
     if os.name == 'nt':
         os.remove(path)
 
+    if tmp_outputfile_fd is not None:
+        tmp_outputfile_fd.close()
+
     return ''.join([syn_rslt, sim_rslt])
 
 
@@ -296,8 +321,14 @@ def run_modelsim(objs, display=False, top=None, outputfile=None,
 
     top_name = top.name if isinstance(top, module.Module) else top
 
+    tmp_outputfile_fd = None
     if outputfile is None:
-        outputfile = 'a.out'
+        tmp_outputfile_fd = tempfile.NamedTemporaryFile(prefix="modelsim_",
+                                                        suffix=".out",
+                                                        dir=os.getcwd())
+        outputfile = tmp_outputfile_fd.name
+
+    outputfile = os.path.abspath(outputfile)
 
     cmd = []
     cmd.append('vlib work ; vmap work ; vlog')
@@ -357,6 +388,9 @@ def run_modelsim(objs, display=False, top=None, outputfile=None,
     if os.name == 'nt':
         os.remove(path)
 
+    if tmp_outputfile_fd is not None:
+        tmp_outputfile_fd.close()
+
     return ''.join([syn_rslt, sim_rslt])
 
 
@@ -489,8 +523,14 @@ def run_verilator(objs, display=False, top=None, outputfile=None,
     if not isinstance(top, module.Module):
         raise ValueError('top module must be specified.')
 
+    tmp_outputfile_fd = None
     if outputfile is None:
-        outputfile = 'obj_dir'
+        tmp_outputfile_fd = tempfile.TemporaryDirectory(prefix="verilator_",
+                                                        suffix=".out",
+                                                        dir=os.getcwd())
+        outputfile = tmp_outputfile_fd.name
+
+    outputfile = os.path.abspath(outputfile)
 
     cmd = []
     cmd.append('verilator')
@@ -571,10 +611,13 @@ def run_verilator(objs, display=False, top=None, outputfile=None,
     if os.name == 'nt':
         cmd.append(outputfile + '/' + 'V' + verilog_prefix)
     else:
-        cmd.append('./' + outputfile + '/' + 'V' + verilog_prefix)
+        cmd.append(outputfile + '/' + 'V' + verilog_prefix)
 
     sim_rslt = _exec(' '.join(cmd), encoding, display)
 
+    if tmp_outputfile_fd is not None:
+        tmp_outputfile_fd.cleanup()
+
     # return ''.join([syn_rslt, make_rslt, sim_rslt])
     return sim_rslt
 
