@@ -129,15 +129,20 @@ class AXIM(axi.AxiMaster, _MutexFunction):
                                                              'read_local_blocksize_buf']),
                                                    self.addrwidth, initval=0)
 
-        self.read_req_idle = self.m.Reg(
-            '_'.join(['', self.name, 'read_req_idle']), initval=1)
-        self.read_data_idle = self.m.Reg(
-            '_'.join(['', self.name, 'read_data_idle']), initval=1)
+        self.read_req_busy = self.m.Reg(
+            '_'.join(['', self.name, 'read_req_busy']), initval=0)
+        self.read_data_busy = self.m.Reg(
+            '_'.join(['', self.name, 'read_data_busy']), initval=0)
 
+        self.read_req_idle = self.m.Wire('_'.join(['', self.name, 'read_req_idle']))
+        self.read_data_idle = self.m.Wire('_'.join(['', self.name, 'read_data_idle']))
         self.read_idle = self.m.Wire('_'.join(['', self.name, 'read_idle']))
-        self.read_idle.assign(
-            vtypes.Ands(vtypes.Not(self.read_start), self.read_req_idle,
-                        self.read_req_fifo.empty, self.read_data_idle))
+
+        self.read_req_idle.assign(vtypes.Ands(vtypes.Not(self.read_start),
+                                              vtypes.Not(self.read_req_busy)))
+        self.read_data_idle.assign(vtypes.Ands(self.read_req_fifo.empty,
+                                               vtypes.Not(self.read_data_busy)))
+        self.read_idle.assign(vtypes.Ands(self.read_req_idle, self.read_data_idle))
 
         self.seq(
             self.read_start(0)
@@ -215,15 +220,20 @@ class AXIM(axi.AxiMaster, _MutexFunction):
                                                               'write_local_blocksize_buf']),
                                                     self.addrwidth, initval=0)
 
-        self.write_req_idle = self.m.Reg(
-            '_'.join(['', self.name, 'write_req_idle']), initval=1)
-        self.write_data_idle = self.m.Reg(
-            '_'.join(['', self.name, 'write_data_idle']), initval=1)
+        self.write_req_busy = self.m.Reg(
+            '_'.join(['', self.name, 'write_req_busy']), initval=0)
+        self.write_data_busy = self.m.Reg(
+            '_'.join(['', self.name, 'write_data_busy']), initval=0)
 
+        self.write_req_idle = self.m.Wire('_'.join(['', self.name, 'write_req_idle']))
+        self.write_data_idle = self.m.Wire('_'.join(['', self.name, 'write_data_idle']))
         self.write_idle = self.m.Wire('_'.join(['', self.name, 'write_idle']))
-        self.write_idle.assign(
-            vtypes.Ands(vtypes.Not(self.write_start), self.write_req_idle,
-                        self.write_req_fifo.empty, self.write_data_idle))
+
+        self.write_req_idle.assign(vtypes.Ands(vtypes.Not(self.write_start),
+                                               vtypes.Not(self.write_req_busy)))
+        self.write_data_idle.assign(vtypes.Ands(self.write_req_fifo.empty,
+                                                vtypes.Not(self.write_data_busy)))
+        self.write_idle.assign(vtypes.Ands(self.write_req_idle, self.write_data_idle))
 
         self.seq(
             self.write_start(0)
@@ -251,26 +261,21 @@ class AXIM(axi.AxiMaster, _MutexFunction):
         global_size = 1
 
         # state 0
-        self.seq.If(fsm.here, self.read_req_idle)(
-            self.read_req_idle(0)
+        self.seq.If(fsm.here, self.read_idle)(
+            self.read_req_busy(1),
+            self.read_data_busy(1)
         )
-        fsm.If(self.read_req_idle).goto_next()
+        fsm.If(self.read_idle).goto_next()
 
         # state 1
         req_cond = fsm.here
         ack = self.read_request(global_addr, global_size, cond=req_cond)
         self.seq.If(fsm.here, ack)(
-            self.read_req_idle(1)
+            self.read_req_busy(0)
         )
         fsm.If(ack).goto_next()
 
         # state 2
-        self.seq.If(fsm.here, self.read_data_idle)(
-            self.read_data_idle(0)
-        )
-        fsm.If(self.read_data_idle).goto_next()
-
-        # state 3
         rcond = fsm.here
         rdata = self.m.TmpReg(self.datawidth, initval=0,
                               signed=True, prefix='axim_rdata')
@@ -279,7 +284,7 @@ class AXIM(axi.AxiMaster, _MutexFunction):
             rdata(self.rdata.rdata)
         )
         self.seq.If(fsm.here, self.rdata.rvalid)(
-            self.read_data_idle(1)
+            self.read_data_busy(0)
         )
         fsm.If(self.rdata.rvalid).goto_next()
 
@@ -292,33 +297,28 @@ class AXIM(axi.AxiMaster, _MutexFunction):
         global_size = 1
 
         # state 0
-        self.seq.If(fsm.here, self.write_req_idle)(
-            self.write_req_idle(0)
+        self.seq.If(fsm.here, self.write_idle)(
+            self.write_req_busy(1),
+            self.write_data_busy(1)
         )
-        fsm.If(self.write_req_idle).goto_next()
+        fsm.If(self.write_idle).goto_next()
 
         # state 1
         req_cond = fsm.here
         ack = self.write_request(global_addr, global_size, cond=req_cond)
         self.seq.If(fsm.here, ack)(
-            self.write_req_idle(1)
+            self.write_req_busy(0)
         )
         fsm.If(ack).goto_next()
 
         # state 2
-        self.seq.If(fsm.here, self.write_data_idle)(
-            self.write_data_idle(0)
-        )
-        fsm.If(self.write_data_idle).goto_next()
-
-        # state 3
         wcond = fsm.here
         wdata = value
         wlast = 1
         _ = self.write_data(wdata, wlast, cond=wcond)
         ack = vtypes.Ors(self.wdata.wready, vtypes.Not(self.wdata.wvalid))
         self.seq.If(fsm.here, ack)(
-            self.write_data_idle(1)
+            self.write_data_busy(0)
         )
         fsm.If(ack).goto_next()
 
@@ -827,7 +827,7 @@ class AXIM(axi.AxiMaster, _MutexFunction):
 
         # Req state 0
         self.seq.If(req_fsm.here, self.read_start)(
-            self.read_req_idle(0)
+            self.read_req_busy(1)
         )
         self.seq.If(self.read_start, self.read_req_fifo.almost_full)(
             self.read_start(1)
@@ -861,7 +861,7 @@ class AXIM(axi.AxiMaster, _MutexFunction):
             cont(0)
         )
         self.seq.If(req_fsm.here, ack, self.read_global_size == 0)(
-            self.read_req_idle(1)
+            self.read_req_busy(0)
         )
         req_fsm.If(ack).goto_init()
 
@@ -898,11 +898,11 @@ class AXIM(axi.AxiMaster, _MutexFunction):
             self.read_data_fsm = data_fsm
 
         # Data state 0
-        cond = vtypes.Ands(self.read_data_idle,
+        cond = vtypes.Ands(vtypes.Not(self.read_data_busy),
                            vtypes.Not(self.read_req_fifo.empty),
                            self.read_op_sel_fifo == op_id)
         self.seq.If(data_fsm.here, cond)(
-            self.read_data_idle(0),
+            self.read_data_busy(1),
             self.read_op_sel_buf(self.read_op_sel_fifo),
             self.read_local_addr_buf(self.read_local_addr_fifo),
             self.read_local_stride_buf(self.read_local_stride_fifo),
@@ -929,7 +929,7 @@ class AXIM(axi.AxiMaster, _MutexFunction):
 
         data_fsm.If(self.rdata.rvalid, self.read_local_size_buf <= 1).goto_init()
         self.seq.If(data_fsm.here, self.rdata.rvalid, self.read_local_size_buf <= 1)(
-            self.read_data_idle(1)
+            self.read_data_busy(0)
         )
 
     def _synthesize_read_data_fsm_narrow(self, ram, port, ram_method, ram_datawidth):
@@ -962,11 +962,11 @@ class AXIM(axi.AxiMaster, _MutexFunction):
             self.read_data_narrow_fsm = data_fsm
 
         # Data state 0
-        cond = vtypes.Ands(self.read_data_idle,
+        cond = vtypes.Ands(vtypes.Not(self.read_data_busy),
                            vtypes.Not(self.read_req_fifo.empty),
                            self.read_op_sel_fifo == op_id)
         self.seq.If(data_fsm.here, cond)(
-            self.read_data_idle(0),
+            self.read_data_busy(1),
             self.read_op_sel_buf(self.read_op_sel_fifo),
             self.read_local_addr_buf(self.read_local_addr_fifo),
             self.read_local_stride_buf(self.read_local_stride_fifo),
@@ -1018,7 +1018,7 @@ class AXIM(axi.AxiMaster, _MutexFunction):
                     count == pack_size - 1).goto_init()
         self.seq.If(data_fsm.here, cond, self.rdata.rvalid, self.read_local_size_buf <= 1,
                     count == pack_size - 1)(
-            self.read_data_idle(1)
+            self.read_data_busy(0)
         )
 
     def _synthesize_read_data_fsm_wide(self, ram, port, ram_method, ram_datawidth):
@@ -1051,11 +1051,11 @@ class AXIM(axi.AxiMaster, _MutexFunction):
             self.read_data_wide_fsm = data_fsm
 
         # Data state 0
-        cond = vtypes.Ands(self.read_data_idle,
+        cond = vtypes.Ands(vtypes.Not(self.read_data_busy),
                            vtypes.Not(self.read_req_fifo.empty),
                            self.read_op_sel_fifo == op_id)
         self.seq.If(data_fsm.here, cond)(
-            self.read_data_idle(0),
+            self.read_data_busy(1),
             self.read_op_sel_buf(self.read_op_sel_fifo),
             self.read_local_addr_buf(self.read_local_addr_fifo),
             self.read_local_stride_buf(self.read_local_stride_fifo),
@@ -1117,11 +1117,11 @@ class AXIM(axi.AxiMaster, _MutexFunction):
                     cond, self.rdata.rvalid, count == 0).goto_init()
         self.seq.If(data_fsm.here, self.read_local_size_buf <= 1,
                     cond, count > 0)(
-            self.read_data_idle(1)
+            self.read_data_busy(0)
         )
         self.seq.If(data_fsm.here, self.read_local_size_buf <= 1,
                     cond, self.rdata.rvalid, count == 0)(
-            self.read_data_idle(1)
+            self.read_data_busy(0)
         )
 
     # --------------------
@@ -1222,7 +1222,7 @@ class AXIM(axi.AxiMaster, _MutexFunction):
 
         # Req state 0
         self.seq.If(req_fsm.here, self.write_start)(
-            self.write_req_idle(0)
+            self.write_req_busy(1)
         )
         self.seq.If(self.write_start, self.write_req_fifo.almost_full)(
             self.write_start(1)
@@ -1271,7 +1271,7 @@ class AXIM(axi.AxiMaster, _MutexFunction):
             cont(0)
         )
         self.seq.If(req_fsm.here, enq_cond, self.write_global_size == 0)(
-            self.write_req_idle(1)
+            self.write_req_busy(0)
         )
         req_fsm.If(enq_cond).goto_init()
 
@@ -1308,11 +1308,11 @@ class AXIM(axi.AxiMaster, _MutexFunction):
             self.write_data_fsm = data_fsm
 
         # Data state 0
-        cond = vtypes.Ands(self.write_data_idle,
+        cond = vtypes.Ands(vtypes.Not(self.write_data_busy),
                            vtypes.Not(self.write_req_fifo.empty),
                            self.write_op_sel_fifo == op_id)
         self.seq.If(data_fsm.here, cond)(
-            self.write_data_idle(0),
+            self.write_data_busy(1),
             self.write_op_sel_buf(self.write_op_sel_fifo),
             self.write_local_addr_buf(self.write_local_addr_fifo),
             self.write_local_stride_buf(self.write_local_stride_fifo),
@@ -1355,7 +1355,7 @@ class AXIM(axi.AxiMaster, _MutexFunction):
 
         data_fsm.If(wcond, rlast).goto_init()
         self.seq.If(data_fsm.here, wcond, rlast)(
-            self.write_data_idle(1)
+            self.write_data_busy(0)
         )
 
     def _synthesize_write_data_fsm_narrow(self, ram, port, ram_method, ram_datawidth):
@@ -1388,11 +1388,11 @@ class AXIM(axi.AxiMaster, _MutexFunction):
             self.write_data_narrow_fsm = data_fsm
 
         # Data state 0
-        cond = vtypes.Ands(self.write_data_idle,
+        cond = vtypes.Ands(vtypes.Not(self.write_data_busy),
                            vtypes.Not(self.write_req_fifo.empty),
                            self.write_op_sel_fifo == op_id)
         self.seq.If(data_fsm.here, cond)(
-            self.write_data_idle(0),
+            self.write_data_busy(1),
             self.write_op_sel_buf(self.write_op_sel_fifo),
             self.write_local_addr_buf(self.write_local_addr_fifo),
             self.write_local_stride_buf(self.write_local_stride_fifo),
@@ -1467,7 +1467,7 @@ class AXIM(axi.AxiMaster, _MutexFunction):
         data_fsm.If(wcond, count == pack_size - 1, wlast).goto_init()
 
         self.seq.If(data_fsm.here, wcond, count == pack_size - 1, wlast)(
-            self.write_data_idle(1)
+            self.write_data_busy(0)
         )
 
     def _synthesize_write_data_fsm_wide(self, ram, port, ram_method, ram_datawidth):
@@ -1500,7 +1500,7 @@ class AXIM(axi.AxiMaster, _MutexFunction):
             self.write_data_wide_fsm = data_fsm
 
         # Data state 0
-        cond = vtypes.Ands(self.write_data_idle,
+        cond = vtypes.Ands(vtypes.Not(self.write_data_busy),
                            vtypes.Not(self.write_req_fifo.empty),
                            self.write_op_sel_fifo == op_id)
         res = vtypes.Mux(
@@ -1509,7 +1509,7 @@ class AXIM(axi.AxiMaster, _MutexFunction):
         local_size = global_size << log_pack_size
 
         self.seq.If(data_fsm.here, cond)(
-            self.write_data_idle(0),
+            self.write_data_busy(1),
             self.write_op_sel_buf(self.write_op_sel_fifo),
             self.write_local_addr_buf(self.write_local_addr_fifo),
             self.write_local_stride_buf(self.write_local_stride_fifo),
@@ -1571,7 +1571,7 @@ class AXIM(axi.AxiMaster, _MutexFunction):
         data_fsm.If(count == pack_size - 1, rvalid, rready, rlast).goto_init()
 
         self.seq.If(data_fsm.here, count == pack_size - 1, rvalid, rready, rlast)(
-            self.write_data_idle(1)
+            self.write_data_busy(0)
         )
 
     def _set_flag(self, fsm, prefix='axim_flag'):
@@ -1717,19 +1717,20 @@ class AXIMVerify(AXIM):
         delay_count = self.m.TmpReg(self.addrwidth, initval=0, prefix='delay_count')
 
         # state 0
-        self.seq.If(fsm.here, self.read_req_idle)(
-            self.read_req_idle(0)
+        self.seq.If(fsm.here, self.read_idle)(
+            self.read_req_busy(1),
+            self.read_data_busy(1)
         )
         fsm(
             delay_count(delay)
         )
-        fsm.If(self.read_req_idle).goto_next()
+        fsm.If(self.read_idle).goto_next()
 
         # state 1
         req_cond = fsm.here
         ack = self.read_request(global_addr, global_size, cond=req_cond)
         self.seq.If(fsm.here, ack)(
-            self.read_req_idle(1)
+            self.read_req_busy(0)
         )
         fsm.If(ack).goto_next()
 
@@ -1737,10 +1738,7 @@ class AXIMVerify(AXIM):
         fsm.If(delay_count > 0)(
             delay_count.dec()
         )
-        self.seq.If(fsm.here, delay_count == 0, self.read_data_idle)(
-            self.read_data_idle(0)
-        )
-        fsm.If(delay_count == 0, self.read_data_idle).goto_next()
+        fsm.If(delay_count == 0).goto_next()
 
         # state 3
         rcond = fsm.here
@@ -1751,7 +1749,7 @@ class AXIMVerify(AXIM):
             rdata(self.rdata.rdata)
         )
         self.seq.If(fsm.here, self.rdata.rvalid)(
-            self.read_data_idle(1)
+            self.read_data_busy(0)
         )
         fsm.If(self.rdata.rvalid).goto_next()
 
@@ -1765,19 +1763,20 @@ class AXIMVerify(AXIM):
         delay_count = self.m.TmpReg(self.addrwidth, initval=0, prefix='delay_count')
 
         # state 0
-        self.seq.If(fsm.here, self.write_req_idle)(
-            self.write_req_idle(0)
+        self.seq.If(fsm.here, self.write_idle)(
+            self.write_req_busy(1),
+            self.write_data_busy(1)
         )
         fsm(
             delay_count(delay)
         )
-        fsm.If(self.write_req_idle).goto_next()
+        fsm.If(self.write_idle).goto_next()
 
         # state 1
         req_cond = fsm.here
         ack = self.write_request(global_addr, global_size, cond=req_cond)
         self.seq.If(fsm.here, ack)(
-            self.write_req_idle(1)
+            self.write_req_busy(0)
         )
         fsm.If(ack).goto_next()
 
@@ -1785,10 +1784,7 @@ class AXIMVerify(AXIM):
         fsm.If(delay_count > 0)(
             delay_count.dec()
         )
-        self.seq.If(fsm.here, delay_count == 0, self.write_data_idle)(
-            self.write_data_idle(0)
-        )
-        fsm.If(delay_count == 0, self.write_data_idle).goto_next()
+        fsm.If(delay_count == 0).goto_next()
 
         # state 3
         wcond = fsm.here
@@ -1797,7 +1793,7 @@ class AXIMVerify(AXIM):
         _ = self.write_data(wdata, wlast, cond=wcond)
         ack = vtypes.Ors(self.wdata.wready, vtypes.Not(self.wdata.wvalid))
         self.seq.If(fsm.here, ack)(
-            self.write_data_idle(1)
+            self.write_data_busy(0)
         )
         fsm.If(ack).goto_next()
 
@@ -1837,7 +1833,8 @@ class AXIMLite(axi.AxiLiteMaster, _MutexFunction):
         fsm.If(ack).goto_next()
 
         # state 1
-        fsm.goto_next()
+        done = vtypes.Ands(self.raddr.arvalid, self.raddr.arready)
+        fsm.If(done).goto_next()
 
         # state 2
         rcond = fsm.here
@@ -1863,7 +1860,8 @@ class AXIMLite(axi.AxiLiteMaster, _MutexFunction):
         fsm.If(ack).goto_next()
 
         # state 1
-        fsm.goto_next()
+        done = vtypes.Ands(self.waddr.awvalid, self.waddr.awready)
+        fsm.If(done).goto_next()
 
         # state 2
         wcond = fsm.here
@@ -1872,13 +1870,18 @@ class AXIMLite(axi.AxiLiteMaster, _MutexFunction):
         ack = vtypes.Ors(self.wdata.wready, vtypes.Not(self.wdata.wvalid))
         fsm.If(ack).goto_next()
 
-    def write_fence(self, fsm, global_addr, value):
-
-        self.write(fsm, global_addr, value)
+        # state 3
+        done = vtypes.Ands(self.wdata.wvalid, self.wdata.wready)
+        fsm.If(done).goto_next()
 
+        # state 4
         res = self.write_completed()
         fsm.If(res).goto_next()
 
+    def write_fence(self, fsm, global_addr, value):
+        """ AXI-Lite Master must not issue any request until the previous request is completed."""
+        self.write(fsm, global_addr, value)
+
     def set_global_base_addr(self, fsm, addr):
 
         if not self.use_global_base_addr:
@@ -1909,12 +1912,16 @@ class AXIMLiteVerify(AXIMLite):
         fsm.If(ack).goto_next()
 
         # state 1
+        done = vtypes.Ands(self.raddr.arvalid, self.raddr.arready)
+        fsm.If(done).goto_next()
+
+        # state 2
         fsm.If(delay_count > 0)(
             delay_count.dec()
         )
         fsm.If(delay_count == 0).goto_next()
 
-        # state 2
+        # state 3
         rcond = fsm.here
         rdata = self.m.TmpReg(self.datawidth, initval=0,
                               signed=True, prefix='axim_rdata')
@@ -1942,14 +1949,26 @@ class AXIMLiteVerify(AXIMLite):
         fsm.If(ack).goto_next()
 
         # state 1
+        done = vtypes.Ands(self.waddr.awvalid, self.waddr.awready)
+        fsm.If(done).goto_next()
+
+        # state 2
         fsm.If(delay_count > 0)(
             delay_count.dec()
         )
         fsm.If(delay_count == 0).goto_next()
 
-        # state 2
+        # state 3
         wcond = fsm.here
         wdata = value
         _ = self.write_data(wdata, cond=wcond)
         ack = vtypes.Ors(self.wdata.wready, vtypes.Not(self.wdata.wvalid))
         fsm.If(ack).goto_next()
+
+        # state 4
+        done = vtypes.Ands(self.wdata.wvalid, self.wdata.wready)
+        fsm.If(done).goto_next()
+
+        # state 5
+        res = self.write_completed()
+        fsm.If(res).goto_next()
