@@ -80,11 +80,14 @@ class AXIStreamOut(axi.AxiStreamOut, _MutexFunction):
                                                    'write_size_buf']),
                                          self.addrwidth + 1, initval=0)
 
-        self.write_data_idle = self.m.Reg(
-            '_'.join(['', self.name, 'write_data_idle']), initval=1)
-
+        self.write_data_busy = self.m.Reg(
+            '_'.join(['', self.name, 'write_data_busy']), initval=0)
+        self.write_data_idle = self.m.Wire('_'.join(['', self.name, 'write_data_idle']))
         self.write_idle = self.m.Wire('_'.join(['', self.name, 'write_idle']))
-        self.write_idle.assign(vtypes.Ands(self.write_req_fifo.empty, self.write_data_idle))
+
+        self.write_data_idle.assign(vtypes.Ands(self.write_req_fifo.empty,
+                                                vtypes.Not(self.write_data_busy)))
+        self.write_idle.assign(self.write_data_idle)
 
         self.write_op_id_map = OrderedDict()
         self.write_op_id_count = 1
@@ -96,10 +99,10 @@ class AXIStreamOut(axi.AxiStreamOut, _MutexFunction):
 
     def write(self, fsm, value, last=False):
         # state 0
-        self.seq.If(fsm.here, self.write_data_idle)(
-            self.write_data_idle(0)
+        self.seq.If(fsm.here, self.write_idle)(
+            self.write_data_busy(1)
         )
-        fsm.If(self.write_data_idle).goto_next()
+        fsm.If(self.write_idle).goto_next()
 
         # state 1
         wcond = fsm.here
@@ -108,7 +111,7 @@ class AXIStreamOut(axi.AxiStreamOut, _MutexFunction):
         _ = self.write_data(wdata, wlast, cond=wcond)
         ack = vtypes.Ors(self.tdata.tready, vtypes.Not(self.tdata.tvalid))
         self.seq.If(fsm.here, ack)(
-            self.write_data_idle(1)
+            self.write_data_busy(0)
         )
         fsm.If(ack).goto_next()
 
@@ -222,11 +225,11 @@ class AXIStreamOut(axi.AxiStreamOut, _MutexFunction):
             self.write_data_fsm = data_fsm
 
         # Data state 0
-        cond = vtypes.Ands(self.write_data_idle,
+        cond = vtypes.Ands(vtypes.Not(self.write_data_busy),
                            vtypes.Not(self.write_req_fifo.empty),
                            self.write_op_sel_fifo == op_id)
         self.seq.If(data_fsm.here, cond)(
-            self.write_data_idle(0),
+            self.write_data_busy(1),
             self.write_op_sel_buf(self.write_op_sel_fifo),
             self.write_local_addr_buf(self.write_local_addr_fifo),
             self.write_local_stride_buf(self.write_local_stride_fifo),
@@ -254,7 +257,7 @@ class AXIStreamOut(axi.AxiStreamOut, _MutexFunction):
 
         data_fsm.If(wcond, rlast).goto_init()
         self.seq.If(data_fsm.here, wcond, rlast)(
-            self.write_data_idle(1)
+            self.write_data_busy(0)
         )
 
     def _synthesize_write_data_fsm_narrow(self, ram, port, ram_method, ram_datawidth):
@@ -285,11 +288,11 @@ class AXIStreamOut(axi.AxiStreamOut, _MutexFunction):
             self.write_data_narrow_fsm = data_fsm
 
         # Data state 0
-        cond = vtypes.Ands(self.write_data_idle,
+        cond = vtypes.Ands(vtypes.Not(self.write_data_busy),
                            vtypes.Not(self.write_req_fifo.empty),
                            self.write_op_sel_fifo == op_id)
         self.seq.If(data_fsm.here, cond)(
-            self.write_data_idle(0),
+            self.write_data_busy(1),
             self.write_op_sel_buf(self.write_op_sel_fifo),
             self.write_local_addr_buf(self.write_local_addr_fifo),
             self.write_local_stride_buf(self.write_local_stride_fifo),
@@ -354,7 +357,7 @@ class AXIStreamOut(axi.AxiStreamOut, _MutexFunction):
         data_fsm.If(wcond, count == pack_size - 1, wlast).goto_init()
 
         self.seq.If(data_fsm.here, wcond, count == pack_size - 1, wlast)(
-            self.write_data_idle(1)
+            self.write_data_busy(0)
         )
 
     def _synthesize_write_data_fsm_wide(self, ram, port, ram_method, ram_datawidth):
@@ -387,7 +390,7 @@ class AXIStreamOut(axi.AxiStreamOut, _MutexFunction):
             self.write_data_wide_fsm = data_fsm
 
         # Data state 0
-        cond = vtypes.Ands(self.write_data_idle,
+        cond = vtypes.Ands(vtypes.Not(self.write_data_busy),
                            vtypes.Not(self.write_req_fifo.empty),
                            self.write_op_sel_fifo == op_id)
         res = vtypes.Mux(
@@ -396,7 +399,7 @@ class AXIStreamOut(axi.AxiStreamOut, _MutexFunction):
         local_size = global_size << log_pack_size
 
         self.seq.If(data_fsm.here, cond)(
-            self.write_data_idle(0),
+            self.write_data_busy(1),
             self.write_op_sel_buf(self.write_op_sel_fifo),
             self.write_local_addr_buf(self.write_local_addr_fifo),
             self.write_local_stride_buf(self.write_local_stride_fifo),
@@ -443,7 +446,7 @@ class AXIStreamOut(axi.AxiStreamOut, _MutexFunction):
         data_fsm.If(count == pack_size - 1, rvalid, rready, rlast).goto_init()
 
         self.seq.If(data_fsm.here, count == pack_size - 1, rvalid, rready, rlast)(
-            self.write_data_idle(1)
+            self.write_data_busy(0)
         )
 
     def _set_flag(self, fsm, prefix='axistreamout_flag'):
@@ -550,11 +553,14 @@ class AXIStreamOutFifo(AXIStreamOut):
                                                    'write_size_buf']),
                                          self.addrwidth + 1, initval=0)
 
-        self.write_data_idle = self.m.Reg(
-            '_'.join(['', self.name, 'write_data_idle']), initval=1)
-
+        self.write_data_busy = self.m.Reg(
+            '_'.join(['', self.name, 'write_data_busy']), initval=0)
+        self.write_data_idle = self.m.Wire('_'.join(['', self.name, 'write_data_idle']))
         self.write_idle = self.m.Wire('_'.join(['', self.name, 'write_idle']))
-        self.write_idle.assign(vtypes.Ands(self.write_req_fifo.empty, self.write_data_idle))
+
+        self.write_data_idle.assign(vtypes.Ands(self.write_req_fifo.empty,
+                                                vtypes.Not(self.write_data_busy)))
+        self.write_idle.assign(self.write_data_idle)
 
         self.write_op_id_map = OrderedDict()
         self.write_op_id_count = 1
@@ -644,11 +650,11 @@ class AXIStreamOutFifo(AXIStreamOut):
             self.write_data_fsm = data_fsm
 
         # Data state 0
-        cond = vtypes.Ands(self.write_data_idle,
+        cond = vtypes.Ands(vtypes.Not(self.write_data_busy),
                            vtypes.Not(self.write_req_fifo.empty),
                            self.write_op_sel_fifo == op_id)
         self.seq.If(data_fsm.here, cond)(
-            self.write_data_idle(0),
+            self.write_data_busy(1),
             self.write_op_sel_buf(self.write_op_sel_fifo),
             self.write_size_buf(self.write_size_fifo),
         )
@@ -692,7 +698,7 @@ class AXIStreamOutFifo(AXIStreamOut):
 
         data_fsm.If(wcond, rlast).goto_init()
         self.seq.If(data_fsm.here, wcond, rlast)(
-            self.write_data_idle(1)
+            self.write_data_busy(0)
         )
 
     def _synthesize_write_data_fsm_narrow(self, fifo, fifo_datawidth):
@@ -722,11 +728,11 @@ class AXIStreamOutFifo(AXIStreamOut):
             self.write_data_narrow_fsm = data_fsm
 
         # Data state 0
-        cond = vtypes.Ands(self.write_data_idle,
+        cond = vtypes.Ands(vtypes.Not(self.write_data_busy),
                            vtypes.Not(self.write_req_fifo.empty),
                            self.write_op_sel_fifo == op_id)
         self.seq.If(data_fsm.here, cond)(
-            self.write_data_idle(0),
+            self.write_data_busy(1),
             self.write_op_sel_buf(self.write_op_sel_fifo),
             # self.write_size_fifo: local_size
             self.write_size_buf(self.write_size_fifo),
@@ -797,7 +803,7 @@ class AXIStreamOutFifo(AXIStreamOut):
         data_fsm.If(wcond, count == pack_size - 1, wlast).goto_init()
 
         self.seq.If(data_fsm.here, wcond, count == pack_size - 1, wlast)(
-            self.write_data_idle(1)
+            self.write_data_busy(0)
         )
 
     def _synthesize_write_data_fsm_wide(self, fifo, fifo_datawidth):
@@ -829,7 +835,7 @@ class AXIStreamOutFifo(AXIStreamOut):
             self.write_data_wide_fsm = data_fsm
 
         # Data state 0
-        cond = vtypes.Ands(self.write_data_idle,
+        cond = vtypes.Ands(vtypes.Not(self.write_data_busy),
                            vtypes.Not(self.write_req_fifo.empty),
                            self.write_op_sel_fifo == op_id)
         res = vtypes.Mux(
@@ -838,7 +844,7 @@ class AXIStreamOutFifo(AXIStreamOut):
         local_size = global_size << log_pack_size
 
         self.seq.If(data_fsm.here, cond)(
-            self.write_data_idle(0),
+            self.write_data_busy(1),
             self.write_op_sel_buf(self.write_op_sel_fifo),
             self.write_size_buf(local_size),
         )
@@ -898,7 +904,7 @@ class AXIStreamOutFifo(AXIStreamOut):
         data_fsm.If(count == pack_size - 1, cur_rvalid, rready, rlast).goto_init()
 
         self.seq.If(data_fsm.here, count == pack_size - 1, cur_rvalid, rready, rlast)(
-            self.write_data_idle(1)
+            self.write_data_busy(0)
         )
 
     def _get_write_op_id(self, fifo):
